import django.contrib.auth.password_validation as validators

from generic_relations.relations import GenericRelatedField
from rest_framework import serializers
from rest_framework.validators import UniqueValidator
from .models import User, BaseAccount
from .factories import serializer_factory


class AccountSerializer(serializers.ModelSerializer):
    username = serializers.CharField(validators=[UniqueValidator(queryset=User.objects.all())])
    email = serializers.CharField(validators=[UniqueValidator(queryset=User.objects.all())], default='')
    account = GenericRelatedField(
        {cls: serializer_factory(cls)() for cls in BaseAccount.__subclasses__()},
        required=False
    )
    password = serializers.CharField(write_only=True)

    class Meta:
        model = User
        exclude = [
            'is_superuser', 'is_staff', 'is_active', 'content_type', 'groups', 'user_permissions', 'account_id',
        ]

    def validate_password(self, value):
        try:
            validators.validate_password(value)
        except Exception as e:
            raise e

        return value

    def get_account_serializer(self, data):
        return self.get_fields()['account'].get_deserializer_for_data(data)

    def create(self, validated_data):
        account_data = validated_data.pop('account', None)

        # Create user
        user = super(AccountSerializer, self).create(validated_data)

        # Encrypt the password
        user.set_password(validated_data['password'])

        # Create account
        if account_data:
            account_serializer = self.get_account_serializer(account_data)
            user.account = account_serializer.Meta.model.objects.create(**account_data)

        user.save()
        return user


class PasswordResetSerializer(serializers.Serializer):
    email = serializers.EmailField()

    def validate_email(self, value):
        if not User.objects.filter(email=value).exists():
            raise serializers.ValidationError('This email does not exist')

        return value


class PasswordUpdateSerializer(serializers.Serializer):
    token = serializers.CharField()
    email = serializers.CharField()
    password = serializers.CharField()

    def validate_email(self, value):
        if not User.objects.filter(email=value).exists():
            raise serializers.ValidationError('This email does not exist')

        return value

    def validate_password(self, value):
        try:
            validators.validate_password(value)
        except:
            return False

        return True



from django.conf import settings
import sys


default_app_config = 'lp_accounts.apps.AccountConfig'

settings.INSTALLED_APPS += [
    'generic_relations',
    'oauth2_provider',
    'social_django',
    'rest_framework_social_oauth2',
]

# Flag for Testing Environment
settings.TESTING = len(sys.argv) > 1 and sys.argv[1] == 'test'

# Extend Native User Model
settings.AUTH_USER_MODEL = 'lp_accounts.User'

# REST Framework Configuration
settings.REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'oauth2_provider.contrib.rest_framework.OAuth2Authentication',
        'rest_framework_social_oauth2.authentication.SocialAuthentication',
    )
}

# Social Auth Configuration
settings.TEMPLATES[0]['OPTIONS']['context_processors'] += [
    'social_django.context_processors.backends',
    'social_django.context_processors.login_redirect',
]
settings.AUTHENTICATION_BACKENDS += [
    'rest_framework_social_oauth2.backends.DjangoOAuth2',
    'django.contrib.auth.backends.ModelBackend',
]

# Configure Templated Email
settings.TEMPLATED_EMAIL_TEMPLATE_DIR = getattr(
    settings, 'TEMPLATED_EMAIL_TEMPLATE_DIR', 'templated_email/')
settings.TEMPLATED_EMAIL_FILE_EXTENSION = getattr(
    settings, 'TEMPLATED_EMAIL_FILE_EXTENSION', 'email'
)

# Welcome Email Configuration
settings.LP_ACCOUNTS_WELCOME_EMAIL_ENABLED = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_ENABLED', False)
settings.LP_ACCOUNTS_WELCOME_EMAIL_TEMPLATE = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_TEMPLATE', 'welcome')
settings.LP_ACCOUNTS_WELCOME_EMAIL_SENDER = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_SENDER', 'support@launchpeer.com')

# Reset Password Email Configuration
settings.LP_ACCOUNTS_PASSWORD_RESET_TEMPLATE = getattr(
    settings, 'LP_ACCOUNTS_FORGOT_PASSWORD_TEMPLATE', 'passwordreset'
)
settings.LP_ACCOUNTS_PASSWORD_RESET_SENDER = getattr(
    settings, 'LP_ACCOUNTS_PASSWORD_RESET_SENDER', 'support@launchpeer.com'
)
